package com.schbrain.canal.client.event;

import com.alibaba.otter.canal.protocol.CanalEntry;
import com.schbrain.canal.client.conf.MethodArgumentConfig;
import com.schbrain.canal.client.core.EditMetaInfo;
import com.schbrain.canal.client.core.MethodArgumentResolver;
import com.schbrain.canal.client.exception.ReflectionException;
import com.schbrain.canal.client.utils.Dml;
import com.schbrain.canal.client.utils.MessageUtil;
import java.lang.reflect.InvocationTargetException;
import java.text.ParseException;
import java.util.List;

public interface ResolverCanalEvent<T> {


    default void onEvent(CanalEntry.Entry entry) throws InvocationTargetException, IllegalAccessException, ParseException, ReflectionException {
        CanalEntry.Header header = entry.getHeader();
        Dml dml = MessageUtil.parse4Dml(entry);
        MethodArgumentResolver resolver = MethodArgumentConfig.LISTENERMETHODARGUMENTRESOLVER;
        List<EditMetaInfo>  metaInfos = resolver.resolver(this,dml);
        CanalEntry.EventType eventType = header.getEventType();
        switch (eventType) {
            case INSERT:
                for (EditMetaInfo metaInfo : metaInfos) {
                    onInsert(header, (T) metaInfo.getAfter());
                }
                break;
            case UPDATE:
                for (EditMetaInfo metaInfo : metaInfos) {
                    onUpdate(header,(T) metaInfo.getBefore(),(T) metaInfo.getAfter());
                }
                break;
            case DELETE:
                for (EditMetaInfo metaInfo : metaInfos) {
                    onDelete(header,(T) metaInfo.getAfter());
                }
                break;
            default:
                break;
        }
    }

    /**
     * onInsert
     * @param t
     */
    void onInsert(CanalEntry.Header header,  T t);

    /**
     * onUpdate
     * @param before
     * @param after
     */
    void onUpdate(CanalEntry.Header header, T before,T after);

    /**
     * onDelete
     * @param t
     */
    void onDelete(CanalEntry.Header header, T  t);
}
